/**---------------------------------------------------------------------------
 ** 
 ** io.h -- 
 ** 
 ** Author          : Pierre Jaccard
 ** Created On      : 1999/07/15 12:38:16
 ** Last Modified By: Pierre Jaccard
 ** Last Modified On: 1999/07/16 20:35:51
 ** Update Count    : 6
 ** Directory       : /home/pego/pcd1/codas3c/gfi/src/libs/nbr/
 ** Version         : 0.0
 ** Status          : Unknown
 ** ---------------------------------------------------------------------- ** 
 ** DESCRIPTION: 
 ** 
 **    See associated file io.c
 ** 
 ** ---------------------------------------------------------------------- ** 
 ** REVISIONS: 
 ** ---------------------------------------------------------------------- ** 
 ** CHANGES: 
 **------------------------------------------------------------------------**/


#ifndef gfilib_nbr_io_already_included
#define gfilib_nbr_io_already_included

#include "geninc.h"
#include "gfimisc.h"
#include "adcp.h"
#include "read.h"

/*
#include <dbext.h>
#include <dbinc.h>
#include <ioserv.h>
#include <mc0.h>
#include <msg_nbr.h>
#include <misc_nbr.h>
#include <read_nbr.h>
*/

#if PROTOTYPE_ALLOWED

int next_nbr_ping(FILE *fp, DATA_FILE_INFO_TYPE *fi, NBR_ENSEMBLE_TYPE *ping,
									char fchg);
int prev_nbr_ping(FILE *fp, DATA_FILE_INFO_TYPE *fi, NBR_ENSEMBLE_TYPE *ping,
									char fchg);
int goto_nbr_ping(FILE *fp, DATA_FILE_INFO_TYPE *fi, NBR_ENSEMBLE_TYPE *ping,
									int num);
int skip_nbr_ping(FILE *fp, DATA_FILE_INFO_TYPE *fi, NBR_ENSEMBLE_TYPE *ping,
									int num);
int init_nbr_ping(FILE *fp, DATA_FILE_INFO_TYPE *fi, NBR_ENSEMBLE_TYPE *ping);

#else

int next_nbr_ping();
int prev_nbr_ping();
int goto_nbr_ping();
int skip_nbr_ping();
int init_nbr_ping();

#endif /* PROTOTYPE_ALLOWED */

#endif /* io_nbr_already_included */
